/*
 * cpp_ls.cpp
 * perturbation with
 *
 *  Created on: Jan 6, 2014
 *      Author: zhou
 */
#include <iostream>
#include <sstream>
#include <fstream>
#include <list>
#include <vector>
#include <algorithm>
#include <set>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <assert.h>
#include <sys/time.h>
#include <unistd.h>
#include "struct.h"

using namespace std;

#define DEBUG 0
#define LOCAL_OPT_RECORD 0
#define MAX_LOCAL_REC_NUM 5000

#define MIN(a,b) ((a)<(b)?(a):(b))
#define PARENT(idx) ((idx-1)/2)
#define LEFT(idx) (2*idx+1)
#define RIGHT(idx) (2*idx+2)
#define EMPTY_IDX 0
const int MAX_CHAR = 10000;
const int MAX_VAL = 999999999;
const float CONST_E = 2.71828f;

//Modified in the script
char param_filename[1000] = "/home/zhou/workspace/org_instances/gauss500-100-4.txt";
int param_knownbest = MAX_VAL;
int param_time = 10;		//the max time for tabu search procedure, unit: second
int param_runcnt = 1;		// the running time for each instance
float param_alpha = 0.1f; 	// the ratio for weak perturbation
float param_beta = 0.4f;	// the raito for strong perturbation
//int param_iter_tabu = 500;
int param_coftabu = 15;	// coefficient of tabu search
const unsigned int param_klen = 4;
int param_tabu_depth = 10000000;
FILE *frec = NULL;

//int param_max_queue_size = 10;
typedef struct STGammaRow{
	int *pos_heap;	//the position of each gamma value
	int *heap;	//the partition index are ordered as a heap according to the corresponding gamma value
	int *values; // the current row
	int size;
}GammaRow;

typedef struct STGammaData{
	int **gammatbl;
	GammaRow* rows;
	int *best_improve;
	int global_best; 
}GammaData;

typedef enum{DESCENT, TABU, DIRECTED} SearchType;

typedef struct ST_Stats{
	float best_found_time;
	int bestresult;
	int *bestpat;
	SearchType found_searh;
	long long globle_itr;
	long long tabu_itr;

	double total_run_time;
	long long total_run_itr;
}Stats;

#if (LOCAL_OPT_RECORD)
typedef struct ST_LC_OPT{
	int *pv;
	int s;
//	bool operator < (const ST_LC_OPT &other){
//		return s > other.s;
//	}
}LC_opt;
LC_opt lc_recs[MAX_LOCAL_REC_NUM];
#endif


int nnode;
int **matrix;
GammaData *pgamma;
Stats finstats;
int lc_num = 0;

/*partition data*/
int *ppos;
int *pbkt;
int pbkt_size = 0;
int *pcnt;
int *pvertex;

int fbest = 0;
int fcurrent = 0;
clock_t starttime;
long long **tabutbl;
long long tabu_itr = 0;
long long gpass = 0;
long long gitr = 0;



inline void recordgitr(){
	gitr++;
//	if (gitr % 10000 == 0){
//		printf("%d,",finstats.bestresult);
//	}
}
void loadCompletedGraph(char *filename){
	ifstream fin;
	fin.open(filename);
	if (fin.fail()){
		cerr << "Can not open file " << filename << endl;
		exit(0);
	}
	fin >> nnode;
	if (fin.eof()){
		cerr << "Empty file" << filename << endl;
		exit(0);
	}
	matrix = new int*[nnode];
	for (int i = 0; i < nnode; i++)
		matrix[i] = new int[nnode];
	int ni = 0;
	int val;
	while (ni < nnode){
		for (int nj = ni; nj < nnode; nj++){
			fin >> val;
			matrix[ni][nj] = matrix[nj][ni] = -val;
		}
		ni++;
	}
}

void printMove(const StepMove *move){
	if (move->mvertex == 1){
		printf("ITR %lld: %d->%d:%d \n",tabu_itr, move->orderedVetexes[0],move->orderedTarget[0], move->inc);
	}
	if (move->mvertex == 2){
		printf("ITR %lld: %d->%d,%d->%d:%d \n",tabu_itr, move->orderedVetexes[0],move->orderedTarget[0],
				move->orderedVetexes[1],move->orderedTarget[1],
				move->inc);
	}
}
void printOderedSteps(const vector<StepMove> *moves, int n = 10){
	for (int i = 0; i < n; i++){
		printMove(&(moves->at(i)));
	}
	printf("\n");
}
/**
 * Calculate F by part
 */
int calculateSum(int *part){
	int sum = 0;
	for (int i = 0 ;i < nnode; i++)
		for (int j = i; j < nnode; j++)
			if (part[i] == part[j])
				sum += matrix[i][j];
	return sum;
}
/**
 * Assign each node to a singleton partition
 */
void allocatePartitionData(){
	ppos = new int[nnode+1];
	pbkt = new int[nnode+1];
	pbkt_size = 0;
	pcnt = new int[nnode+1];
	pvertex = new int[nnode];
	for (int i = 0; i < nnode+1; i++){
		ppos[i] = i;
		pbkt[i] = i;
	}
	pbkt_size = 1;
	memset(pcnt, 0, sizeof(int) * (nnode+1));
	memset(pvertex, -1, sizeof(int) * nnode);
}

void disposePartition(){
	delete[] ppos;
	delete[] pbkt;
	delete[] pcnt;
	delete[] pvertex;
}
void buildPartition(int *vpart){
	for (int i = 0; i < nnode; i++){
		int pid = vpart[i];
		if (pid == EMPTY_IDX || pid > nnode){
			printf("Unvalid parition for node %d\n",pid);
			exit(0);
		}
		if (ppos[pid] >= pbkt_size){ //
			//ASSERTATION
			if (pbkt_size == nnode+1){
				printf("The bucket is full, no new partition could be added\n");
				exit(0);
			}
			int end_pid = pbkt[pbkt_size];
			swapAry(pbkt, pbkt_size, ppos[pid]);
			swapAry(ppos, end_pid, pid);
			pbkt_size++;
			pcnt[pid]++;
		}else{
			pcnt[pid]++;
		}
	}
	memcpy(pvertex, vpart, sizeof(int)*nnode);
}
int calculateDistance(int *p1, int *p2){
	int sum = 0;
	for (int i = 0; i < nnode; i++){
		for (int j = i+1; j < nnode; j++){
			if (p1[i] == p1[j]){
				if (p2[i] != p2[j])
					sum++;
			}else{
				if (p2[i] == p2[j])
					sum++;
			}
		}
	}
	return sum;
}


/*****************************Data Structure of Gamma Table*********************************************/
/**
 * The gamma table have already updated before invoking this function
 */
void adjustGammaRow(GammaRow *gr, int partid){
	int curidx = gr->pos_heap[partid];
	int *v = gr->values;
	assert(curidx < gr->size);
	//ascend
	int parent = PARENT(curidx);
	if (parent >= 0 && v[gr->heap[parent]] < v[gr->heap[curidx]]){
		do{
			swapAry(gr->pos_heap, gr->heap[parent], gr->heap[curidx]);
			swapAry(gr->heap, parent, curidx);
			curidx = parent;
			parent = PARENT(curidx);
		}while (parent >= 0 && v[gr->heap[parent]] < v[gr->heap[curidx]]);
	}else{ //descend
		int end = 0;
		int biggeridx = curidx;
		while (!end){
			int left = LEFT(curidx);
			int right = RIGHT(curidx);
			if (left < gr->size && v[gr->heap[left]] > v[gr->heap[biggeridx]]){
				biggeridx = left;
			}
			if (right < gr->size && v[gr->heap[right]] > v[gr->heap[biggeridx]]){
				biggeridx = right;
			}
			if (biggeridx != curidx){
				swapAry(gr->pos_heap, gr->heap[biggeridx], gr->heap[curidx]);
				swapAry(gr->heap, biggeridx, curidx);
				curidx = biggeridx;
			}else{
				end = 1;
			}
		}
	}
}

void buildGammaRowFromPartition(GammaRow *gr,int *values, int len){
	int curpart = 0;
	gr->values = values;
	gr->pos_heap = new int[len];
	gr->heap = new int[len];
	gr->size = 0;
	memset(gr->pos_heap, -1, sizeof(int) * len);
	memset(gr->heap, -1, sizeof(int) * len);

	while (gr->size < pbkt_size){
		curpart = pbkt[gr->size];
		gr->heap[gr->size] = curpart;
		gr->pos_heap[curpart] = gr->size;
		gr->size++;
		adjustGammaRow(gr, curpart);
//		printRow(gr);
	}
}
void dropGammaRow(GammaRow *gr, int partid){
	int curidx = gr->pos_heap[partid];

	swapAry(gr->pos_heap, partid, gr->heap[gr->size-1]);
	swapAry(gr->heap, curidx, gr->size-1);
	gr->pos_heap[partid] = -1;
	gr->size--;
	if (curidx != gr->size)
		adjustGammaRow(gr, gr->heap[curidx]);
}

void addGammaRow(GammaRow *gr, int partid){
	int curidx = gr->size;
	gr->heap[curidx] = partid;
	gr->pos_heap[partid] = curidx;
	gr->size++;
	adjustGammaRow(gr, partid);
}

GammaData* buildGammaData(){
	GammaData* gamma = new GammaData;
	gamma->gammatbl = new int*[nnode];
	gamma->best_improve = new int[nnode];
	gamma->rows = new GammaRow[nnode];
	memset(gamma->best_improve, -1, sizeof(int) * nnode);
	//for each node
	for(int i = 0; i < nnode; i++){
		gamma->gammatbl[i] = new int[nnode+1];
		memset(gamma->gammatbl[i], 0, sizeof(int) * (nnode+1));
		for (int j = 0; j < nnode; j++){
			gamma->gammatbl[i][pvertex[j]] += matrix[i][j];
		}
		buildGammaRowFromPartition(&(gamma->rows[i]), gamma->gammatbl[i], nnode+1);
		int top_part = gamma->rows[i].heap[0];
		int selft_part = pvertex[i];
		GammaRow *grow =  &(gamma->rows[i]);
	//ERROR: The nodes which does not meet the following condition will NOT be initialized
	//In order to compare, we did not remove this error
//		if (gamma->gammatbl[i][best_part] > gamma->gammatbl[i][selft_part])
//			gamma->best_improve[i] = best_part;
		if (selft_part != top_part)
			gamma->best_improve[i] = top_part;
		else{
			int left = grow->heap[1];
			gamma->best_improve[i] = left;
			if (grow->size > 2 ){
				if (grow->values[grow->heap[2]] > grow->values[left])
					gamma->best_improve[i] = grow->heap[2];
			}
		}
	}
	return gamma;
}

/*Move vertex from src_part to dest_part, call this function before execute the move */
void updateGamma(GammaData *gamma, int vertex, int src_part, int dest_part){
	//move vertex i to partition
	assert(pvertex[vertex] == src_part);
	for (int i = 0; i < nnode; i++){
		GammaRow *grow = &(gamma->rows[i]);
		int *v = grow->values;
		//update the gamma table
		gamma->gammatbl[i][src_part] -= matrix[vertex][i];
		gamma->gammatbl[i][dest_part] += matrix[vertex][i];
		if (pcnt[src_part] == 1){
			dropGammaRow(grow, src_part);
		}else
			adjustGammaRow(grow, src_part);

//		printRow(&(gamma->rows[i]));
		if (pcnt[dest_part] == 0){
			addGammaRow(grow, dest_part);
		}else{
			adjustGammaRow(grow, dest_part);
		}
		//adjust the best
		int self_part = (i == vertex) ? dest_part : pvertex[i];
		int top = grow->heap[0];
		if (self_part != top)
			gamma->best_improve[i] = top;
		else{
			int left = grow->heap[1];
			gamma->best_improve[i] = left;
			if (grow->size > 2 ){
				if (v[grow->heap[2]] > v[left])
					gamma->best_improve[i] = grow->heap[2];
			}
		}
	}
}
void disposeGammaRow(GammaRow *gr){
	delete[] gr->heap;
	delete[] gr->pos_heap;
}
void disposeGamma(GammaData *gamma){
	for (int i = 0; i < nnode; i++){
		delete[] gamma->gammatbl[i];
		disposeGammaRow(gamma->rows+i);
	}
	delete[] gamma->rows;
	delete[] gamma->gammatbl;
	delete[] gamma->best_improve;
}

/**************************Other useful funtion****************************/
int compareSolution(int *sa, int *sb, int n){
	int *dict = new int[n];
	memset(dict, -1, sizeof(int) * n);
	for (int i = 0; i < n; i++){
		if (dict[sa[i]] == -1)
			dict[sa[i]] = sb[i];
		else
			if (sb[i] != dict[sa[i]])
				return 0;
	}
	delete[] dict;
	return 1;
}


void updatePartition(int v, int target){
	int oldpartition = pvertex[v];
	assert(target !=EMPTY_IDX);
	if (oldpartition == target)
		printf("Meaningless move\n");
	pcnt[oldpartition]--;
	pcnt[target]++;
	//the old cluster of v is empty
	if (pcnt[oldpartition] == 0){
		int end_pid = pbkt[pbkt_size-1];
		swapAry(pbkt, pbkt_size-1, ppos[oldpartition]);
		swapAry(ppos, end_pid, oldpartition);
		pbkt_size--;
	}
	pvertex[v] = target;
}

int decideTarget(StepMove move){
	int target = move.orderedTarget[0];
	assert(move.mvertex == 1);
	assert(ppos[target] < pbkt_size);
	//Move v to a new cluster
	if (target == EMPTY_IDX){
		assert(pbkt_size > 0);
		target = pbkt[pbkt_size];
		pbkt_size++;
	}
	return target;
}
/**
 * update partition, partcnt, tabu,move freq
 */
void executeStepMove(StepMove &move){
	int target = decideTarget(move);
	int curvertex = move.orderedVetexes[0];

	//update gamma table
	updateGamma(pgamma, curvertex, pvertex[curvertex], target);
	updatePartition(curvertex, target);
	fcurrent += move.inc;
}
void recordBest(SearchType srh){
	fbest = fcurrent;

	finstats.bestresult = fbest;
	finstats.best_found_time = (float)(clock() - starttime) / CLOCKS_PER_SEC;
	finstats.globle_itr = gpass;
	finstats.found_searh = srh;
	if (srh == TABU){
		finstats.tabu_itr = tabu_itr;
	}
	memcpy(finstats.bestpat, pvertex, sizeof(int)*nnode);
	finstats.total_run_itr = 0;
}
void clearFinalStats(){
	finstats.best_found_time = 0.0f;
	finstats.bestresult = -MAX_VAL;
	finstats.globle_itr = 0l;
	finstats.tabu_itr = 0l;
	finstats.total_run_time = 0.0f;
	if (finstats.bestpat == NULL)
		finstats.bestpat = new int[nnode];
	memset(finstats.bestpat, 0, sizeof(int) * nnode);
	finstats.total_run_itr = 0;
}
void initDataStructure(){
	allocatePartitionData();
	clearFinalStats();
	int *initpart = new int[nnode];
	for (int i = 0; i < nnode; i++){
		initpart[i] = i+1;
	}
	buildPartition(initpart);
	fbest = fcurrent =0;
	tabutbl = new long long*[nnode];// for moving two nodes
	for (int i = 0; i < nnode; i++){
		tabutbl[i] = new long long[nnode+1];
		memset(tabutbl[i], -1, sizeof(long long) * (nnode+1));
	}
	pgamma = buildGammaData();

	starttime = clock();
	tabu_itr = 0;
	gpass = 0;
#if (LOCAL_OPT_RECORD)
	for (int i = 0; i < MAX_LOCAL_REC_NUM; i++){
		lc_recs[i].pv = NULL;
		lc_recs[i].s = 0;
	}
	lc_num = 0;
#endif
}

/**
 * Dispose gamma, tabutbl, partitiondata
 */
void clearDataStructure(){
	disposeGamma(pgamma);
	disposePartition();
	for (int i = 0; i < nnode; i++){
			delete[] tabutbl[i];
	}
	delete[] tabutbl;
}
void descentSearch(){
	int *randlst = new int[nnode];
	int improved = 1;

	while (improved == 1){
		improved = 0;
		generateRandList(randlst, nnode);
		for (int i = 0; i < nnode; i++){
			int currentnode = randlst[i];
			int best_part = pgamma->best_improve[currentnode];
			int gain = pgamma->gammatbl[currentnode][best_part] - pgamma->gammatbl[currentnode][pvertex[currentnode]];
			if (gain > 0){
				improved = 1;
				StepMove sm = make_1StepMove(currentnode, best_part, gain);
				executeStepMove(sm);
			}
		}
		recordgitr();
	}
	assert(calculateSum(pvertex) == fcurrent);
	if (fcurrent > fbest){
		recordBest(DESCENT);
	}
//	printf("After descent: current %d-%d \n",gitr, fcurrent);
}

int findBestMove(StepMove *chosenMove){
	StepMove maxmove;
	int bestinc = -MAX_VAL;
	int eqcnt = 0;
	for (int nodeidx = 0; nodeidx < nnode; nodeidx++){
		int best_part = pgamma->best_improve[nodeidx];
		assert(best_part != pvertex[nodeidx]);
		if (best_part == EMPTY_IDX && pcnt[pvertex[nodeidx]] == 1)
			continue; //meaningless move
		int gain = pgamma->gammatbl[nodeidx][best_part] - pgamma->gammatbl[nodeidx][pvertex[nodeidx]];
		StepMove curmove = make_1StepMove(nodeidx, best_part, gain);
		if (fcurrent + gain > fbest || tabutbl[nodeidx][best_part] < tabu_itr){
			if (gain > bestinc){
				bestinc = gain;
				maxmove = curmove;
				eqcnt = 0;
			}
			else if (gain == bestinc){ // Find the best one from all the moves with negative improvement;
				eqcnt++;
				if (rand() % eqcnt == 0)
					maxmove = curmove;
			}
		}
	}
	if (bestinc == -MAX_VAL)
		return 0;
	memcpy(chosenMove, &maxmove, sizeof(StepMove));
	return 1;
}

void tabuExploreSearch(){
	int non_improve_cnt = 0;
#if (LOCAL_OPT_RECORD)
	int local_opt = 0;
	int *plocal_s = new int[nnode];
#endif
	while (non_improve_cnt < nnode ){
		StepMove chosenMove;
		int find = findBestMove(&chosenMove);
		if (find == 0){
			cerr << "None node could be moved\n" << endl;
			exit(0);
		}
		int curvertex = chosenMove.orderedVetexes[0];
		int oldpart = pvertex[curvertex];
		/*Update tabu table*/
		if (pcnt[oldpart] == 1)
			tabutbl[curvertex][EMPTY_IDX] = tabu_itr + MIN(param_coftabu, nnode / 4) + rand() % pbkt_size;
		// if there are only two nodes in a cluster and an edge will be moved outside, the cluster will be clean out
		//else if (pcnt[oldpart] == 2 && move.mvertex == 2)
		//	tabutbl[curvertex][EMPTY_IDX] = tabutenure_base + rand() % pbkt_size;
		else
			tabutbl[curvertex][oldpart] = tabu_itr + MIN(param_coftabu, nnode / 4) + rand() % pbkt_size;
		executeStepMove(chosenMove);
		if (fcurrent > fbest){
			recordBest(TABU);
			non_improve_cnt = 0;
			if (fbest >= param_knownbest){
				break;
			}
		}else
			non_improve_cnt++;
#if LOCAL_OPT_RECORD
		if (fcurrent > local_opt){
			local_opt = fcurrent;
			memcpy(plocal_s, pvertex, sizeof(int)*nnode);
		}
#endif
		recordgitr();
		tabu_itr++;
	}
	tabu_itr = tabu_itr + MIN(param_coftabu, nnode / 4) + nnode; //jump D step to invalid the data in tabu table
//	printf("After tabu: current %d-%d \n",gitr, fcurrent);
	/*record local optimum*/
#if LOCAL_OPT_RECORD
	if (lc_num < MAX_LOCAL_REC_NUM){
		lc_recs[lc_num].pv = plocal_s;
		lc_recs[lc_num].s = local_opt;
		lc_num++;
	}
#endif
}
/**
 * *strength 
 * tabuitr
 */
void directedPerturb(){
	int *moved = new int[nnode];
	memset(moved, 0, sizeof(int) * nnode);
//	int param_max_queue_size =  10 + random() % 10;
	int param_max_queue_size = 10 + rand() % (int)(0.05* nnode + 1);
#if (DEBUG)
	printf("PERTURBATION starts from %lldth\n",tabu_itr);
#endif
//	strength = param_alpha  * nnode + rand() % (int((param_beta - param_alpha) * nnode));
	//Ensure the strenth is less than the number of nodes;
	int strength = (int)(param_alpha * nnode) + rand() % (int)(param_beta* nnode);
	int itr_cnt = 0;
	while(itr_cnt < strength){
		FixedSizeQueue* fq = newFixedSizeQueue(param_max_queue_size);
		for (int i = 0; i < nnode; i++){
			int best_part = pgamma->best_improve[i];
			if (moved[i] == 1 && (best_part != 0 || (best_part == 0 && pcnt[pvertex[i]]>1) ))
				continue;
			int gain = pgamma->gammatbl[i][best_part] - pgamma->gammatbl[i][pvertex[i]];
			StepMove sm = make_1StepMove(i, best_part, gain);
			insertFixedSizeQueue(fq, sm);
		}
		// chose a random move in the candidate queue
		StepMove choice = randSelect(fq);
		//execute the move
		executeStepMove(choice);
		if (fcurrent > fbest){
			recordBest(DIRECTED);
		}
#if (DEBUG)
		printMove(&choice);
#endif
		moved[choice.orderedVetexes[0]] = 1;
		itr_cnt++;
		recordgitr();
	}
#if (DEBUG)
	printf("PERTURBATION ends at %lldth\n",gpass);
#endif
	delete[] moved;
//	printf("After directed: current %d-%d \n",gitr, fcurrent);
}


void threePhaseMain(int index){
//	int perturb_itr = 0;
//	int end = 0;
//	//DEBUG time
//	struct timeval tabustart,tabuend;
//	struct timeval pstart,pend;
//	long long psum=0,tabusum=0;

	gpass = 0;
	int len = 0;
	initDataStructure();
	//gitr < 10000000
#if (LOCAL_OPT_RECORD)
	while ((double)(clock() - starttime) / CLOCKS_PER_SEC < (double)param_time || lc_num < MAX_LOCAL_REC_NUM){
#else
	while ((double)(clock() - starttime) / CLOCKS_PER_SEC < (double)param_time){
#endif

		descentSearch();
		tabuExploreSearch();
		directedPerturb();
		if (lc_num == MAX_LOCAL_REC_NUM && fbest >= param_knownbest){
			break;
		}
//		if (gpass % 100 == 0){
//			printf("%d,", finstats.bestresult);
//			fflush(stdout);
//			len++;
//		}
//		gpass++;
	}
	finstats.total_run_time = (double)(clock() - starttime) / CLOCKS_PER_SEC;
	finstats.total_run_itr = gpass;
	clearDataStructure();
}



int countPartition(int *p){
	int npart = 0;
	//check
	int *cnt = new int[nnode];
	memset(cnt, 0, sizeof(int) * nnode);
	for (int i = 0; i < nnode; i++)
		cnt[p[i]]++;
	for (int i = 0; i < nnode; i++)
		if (cnt[i] != 0)
			npart++;
	return npart;
}

void showUsage(){
	//TODO:
	cout << "Wrong parameter" << endl;
}
void readParameters(int argc, char **argv){
	for (int i = 1; i < argc; i+=2){
		if (argv[i][0] != '-' || argv[i][2] != 0){
			showUsage();
			exit(0);
		}else if (argv[i][1] == 'f'){
			strncpy(param_filename, argv[i+1],1000);
		}else if (argv[i][1] == 's'){
			param_time = atoi(argv[i+1]);
		}else if(argv[i][1] == 'a'){
			param_alpha = atof(argv[i+1]);
		}else if (argv[i][1] == 'b'){
			param_beta =  atof(argv[i+1]);
		}else if (argv[i][1] == 't'){
			param_coftabu = atoi(argv[i+1]);
		}else if (argv[i][1] == 'r'){
			param_runcnt = atoi(argv[i+1]);
		}
	}
	/*check parameters*/
	if (strlen(param_filename) == 0){
		cerr << "No input data" << endl;
		exit(1);
	}
}


int setupRecordFile(){
	/*creat file in current direct*/
	char path_cwd[FILENAME_MAX];
	char *graph_name = basename(param_filename);
	char file_name[FILENAME_MAX];
	time_t timer = time(NULL);

	getcwd(path_cwd, FILENAME_MAX);
//	sprintf(file_name, "%s/%s.%d-rec",path_cwd, graph_name,timer);

	sprintf(file_name, "%s/%s_best",path_cwd, graph_name);
	frec = fopen(file_name, "a+");
	if (frec == NULL){
		return 0;
	}
	return 1;
}

void showPartition(FILE *f, int *p){
	int *mark = new int[nnode+1];
	memset(mark, 0, sizeof(int) * (nnode+1));
	for (int i = 0; i < nnode; i++) mark[p[i]] = 1;
	int cnt = 1;
	for (int i = 0; i < nnode + 1; i++){
		if (mark[i] != 0)
			mark[i] = cnt++;
	}
	for(int i = 0; i < nnode; i++){
		fprintf(f, "%d ",mark[p[i]]);
	}
	fprintf(f, "\n");
}
int main_entry(int argc, char **argv){
	/*displat and read parameters*/
	for (int i = 0; i < argc; i++){
		fprintf(stdout, "%s ", argv[i]);
	}
	fprintf(stdout, "\n");
	readParameters(argc, argv);

	//set logging device
	if (0 == setupRecordFile()){
		cerr << "Failed in open log file" << endl;
		exit(2);
	}

	//load graph by jostle format
	loadCompletedGraph(param_filename);
	/*initialize some random parameter*/
	srand((unsigned int)time(NULL));
	fprintf(stdout, "file %s;  time %d; alpha %.2f; beta %.2f; tabucof %d; param_runcnt %d\n", \
			param_filename, param_time, param_alpha, param_beta,  param_coftabu, param_runcnt);

	fprintf(stdout, "index\t best_obj\t found_time\t best_itr\t tabu_itr\t patnum\t  total_time\t total_itr\n");
	int cnt = 0;
	float sumtime = 0;
	int sumres = 0;
	long long sumiter = 0;

	int bestInAll = -MAX_VAL;
	int *bestInAlllPartition = new int[nnode];
	while (cnt < param_runcnt){
		threePhaseMain(cnt);
		int checked = calculateSum(finstats.bestpat);
		if (checked != finstats.bestresult){
			fprintf(stderr, "fatal error, result does not match %d %d", finstats.bestresult, checked);
			exit(1);
		}
		fprintf(stdout, "%d\t %d\t %.2f\t %lld\t %lld\t %d\t %.2f %lld\n",cnt+1, finstats.bestresult, finstats.best_found_time,
				 finstats.globle_itr, finstats.tabu_itr, countPartition(finstats.bestpat),finstats.total_run_time,finstats.total_run_itr);
		if (finstats.bestresult > bestInAll){
			bestInAll = finstats.bestresult;
			memcpy(bestInAlllPartition, finstats.bestpat, sizeof(int) * nnode);
		}
		cnt++;
		sumtime += finstats.best_found_time;
		sumres += finstats.bestresult;
		sumiter += finstats.globle_itr;
#if (LOCAL_OPT_RECORD)
		/*clear local optimum*/
		fprintf(frec, "local optimums number %d. gloabl best %d, Format(dis, fit)\n",lc_num, bestInAll);
		for (int i = 0; i < lc_num; i++){
			int di = calculateDistance(lc_recs[i].pv, finstats.bestpat);
			double nor_di = (double)di *2/ (nnode *(nnode-1));
			fprintf(frec, "%d, %.4f, %d\n", di, nor_di, lc_recs[i].s);
			delete lc_recs[i].pv;
		}
		lc_num = 0;
#endif
	}
	fprintf(stdout,"best result: %d\n",bestInAll);
	fprintf(stdout, "average time:%f\n", sumtime / param_runcnt);
	fprintf(stdout, "average result:%f\n", (float)sumres/param_runcnt);
	fprintf(stdout, "average best iteration: %f\n", (float)sumiter/param_runcnt);

	fprintf(frec, "%d\n", bestInAll);
	showPartition(frec, bestInAlllPartition);
	fclose(frec);
	return 0;
}

int main(int argc, char **argv){
	main_entry(argc, argv);
}
